@@ -33,6 +33,7 @@ enum {
 struct tt {
 	size_t	ncols;		/* number of columns */
 	size_t	termwidth;	/* terminal width */
+	int	is_term;	/* is a tty? */
 	int	flags;
 	int	first_run;
 
@@ -87,7 +88,7 @@ extern struct tt_line *tt_add_line(struct tt *tb, struct tt_line *parent);
 extern int tt_line_set_data(struct tt_line *ln, int colnum, const char *data);
 extern int tt_line_set_userdata(struct tt_line *ln, void *data);
 
-extern int tt_parse_columns_list(const char *list, int ary[], size_t arrsz,
-				int (name2id)(const char *, size_t));
+extern void tt_fputs_quoted(const char *data, FILE *out);
+extern void tt_fputs_nonblank(const char *data, FILE *out);
 
 #endif /* UTIL_LINUX_TT_H */
