@@ -37,30 +37,7 @@
 #include "at.h"
 #include "sysfs.h"
 
-char *blkid_strndup(const char *s, int length)
-{
-	char *ret;
-
-	if (!s)
-		return NULL;
-
-	if (!length)
-		length = strlen(s);
-
-	ret = malloc(length + 1);
-	if (ret) {
-		strncpy(ret, s, length);
-		ret[length] = '\0';
-	}
-	return ret;
-}
-
-char *blkid_strdup(const char *s)
-{
-	return blkid_strndup(s, 0);
-}
-
-char *blkid_strconcat(const char *a, const char *b, const char *c)
+static char *blkid_strconcat(const char *a, const char *b, const char *c)
 {
 	char *res, *p;
 	size_t len, al, bl, cl;
@@ -103,7 +80,8 @@ static void add_to_dirlist(const char *dir, const char *subdir,
 	if (!dp)
 		return;
 	dp->name = subdir ? blkid_strconcat(dir, "/", subdir) :
-			    blkid_strdup(dir);
+		   dir ? strdup(dir) : NULL;
+
 	if (!dp->name) {
 		free(dp);
 		return;
@@ -137,7 +115,7 @@ void blkid__scan_dir(char *dirname, dev_t devno, struct dir_list **list,
 	if ((dir = opendir(dirname)) == NULL)
 		return;
 
-	while ((dp = readdir(dir)) != 0) {
+	while ((dp = readdir(dir)) != NULL) {
 #ifdef _DIRENT_HAVE_D_TYPE
 		if (dp->d_type != DT_UNKNOWN && dp->d_type != DT_BLK &&
 		    dp->d_type != DT_LNK && dp->d_type != DT_DIR)
@@ -153,8 +131,7 @@ void blkid__scan_dir(char *dirname, dev_t devno, struct dir_list **list,
 
 		if (S_ISBLK(st.st_mode) && st.st_rdev == devno) {
 			*devname = blkid_strconcat(dirname, "/", dp->d_name);
-			DBG(DEBUG_DEVNO,
-			    printf("found 0x%llx at %s\n", (long long)devno,
+			DBG(DEVNO, blkid_debug("found 0x%llx at %s", (long long)devno,
 				   *devname));
 			break;
 		}
@@ -197,16 +174,7 @@ static const char *devdirs[] = { "/devices", "/devfs", "/dev", NULL };
  * @short_description: mix of various utils for low-level and high-level API
  */
 
-/* returns basename and keeps dirname in the @path */
-static char *stripoff_last_component(char *path)
-{
-	char *p = strrchr(path, '/');
 
-	if (!p)
-		return NULL;
-	*p = '\0';
-	return ++p;
-}
 
 static char *scandev_devno_to_devpath(dev_t devno)
 {
@@ -225,7 +193,7 @@ static char *scandev_devno_to_devpath(dev_t devno)
 		struct dir_list *current = list;
 
 		list = list->next;
-		DBG(DEBUG_DEVNO, printf("directory %s\n", current->name));
+		DBG(DEVNO, blkid_debug("directory %s", current->name));
 		blkid__scan_dir(current->name, devno, &new_list, &devname);
 		free(current->name);
 		free(current);
@@ -268,44 +236,15 @@ char *blkid_devno_to_devname(dev_t devno)
 		path = scandev_devno_to_devpath(devno);
 
 	if (!path) {
-		DBG(DEBUG_DEVNO,
-		    printf("blkid: couldn't find devno 0x%04lx\n",
+		DBG(DEVNO, blkid_debug("blkid: couldn't find devno 0x%04lx",
 			   (unsigned long) devno));
 	} else {
-		DBG(DEBUG_DEVNO,
-		    printf("found devno 0x%04llx as %s\n", (long long)devno, path));
+		DBG(DEVNO, blkid_debug("found devno 0x%04llx as %s", (long long)devno, path));
 	}
 
 	return path;
 }
 
-static int get_dm_wholedisk(struct sysfs_cxt *cxt, char *diskname,
-			    size_t len, dev_t *diskdevno)
-{
-	int rc = 0;
-	char *name;
-
-	/* Note, sysfs_get_slave() returns the first slave only,
-	 * if there is more slaves, then return NULL
-	 */
-	name = sysfs_get_slave(cxt);
-	if (!name)
-		return -1;
-
-	if (diskname && len) {
-		strncpy(diskname, name, len);
-		diskname[len - 1] = '\0';
-	}
-
-	if (diskdevno) {
-		*diskdevno = sysfs_devname_to_devno(name, NULL);
-		if (!*diskdevno)
-			rc = -1;
-	}
-
-	free(name);
-	return rc;
-}
 
 /**
  * blkid_devno_to_wholedisk:
@@ -346,99 +285,7 @@ static int get_dm_wholedisk(struct sysfs_cxt *cxt, char *diskname,
 int blkid_devno_to_wholedisk(dev_t dev, char *diskname,
 			size_t len, dev_t *diskdevno)
 {
-	struct sysfs_cxt cxt;
-	int is_part = 0;
-
-	if (!dev || sysfs_init(&cxt, dev, NULL) != 0)
-		return -1;
-
-	is_part = sysfs_has_attribute(&cxt, "partition");
-	if (!is_part) {
-		/*
-		 * Extra case for partitions mapped by device-mapper.
-		 *
-		 * All regualar partitions (added by BLKPG ioctl or kernel PT
-		 * parser) have the /sys/.../partition file. The partitions
-		 * mapped by DM don't have such file, but they have "part"
-		 * prefix in DM UUID.
-		 */
-		char *uuid = sysfs_strdup(&cxt, "dm/uuid");
-		char *tmp = uuid;
-		char *prefix = uuid ? strsep(&tmp, "-") : NULL;
-
-		if (prefix && strncasecmp(prefix, "part", 4) == 0)
-			is_part = 1;
-		free(uuid);
-
-		if (is_part &&
-		    get_dm_wholedisk(&cxt, diskname, len, diskdevno) == 0)
-			/*
-			 * partitioned device, mapped by DM
-			 */
-			goto done;
-
-		is_part = 0;
-	}
-
-	if (!is_part) {
-		/*
-		 * unpartitioned device
-		 */
-		if (diskname && len) {
-			if (!sysfs_get_devname(&cxt, diskname, len))
-				goto err;
-		}
-		if (diskdevno)
-			*diskdevno = dev;
-
-	} else {
-		/*
-		 * partitioned device
-		 *	- readlink /sys/dev/block/8:1   = ../../block/sda/sda1
-		 *	- dirname  ../../block/sda/sda1 = ../../block/sda
-		 *	- basename ../../block/sda      = sda
-		 */
-		char linkpath[PATH_MAX];
-		char *name;
-		int linklen;
-
-		linklen = sysfs_readlink(&cxt, NULL,
-				linkpath, sizeof(linkpath) - 1);
-		if (linklen < 0)
-			goto err;
-		linkpath[linklen] = '\0';
-
-		stripoff_last_component(linkpath);		/* dirname */
-		name = stripoff_last_component(linkpath);	/* basename */
-		if (!name)
-			goto err;
-
-		if (diskname && len) {
-			strncpy(diskname, name, len);
-			diskname[len - 1] = '\0';
-		}
-
-		if (diskdevno) {
-			*diskdevno = sysfs_devname_to_devno(name, NULL);
-			if (!*diskdevno)
-				goto err;
-		}
-	}
-
-done:
-	sysfs_deinit(&cxt);
-
-	DBG(DEBUG_DEVNO,
-	    printf("found entire diskname for devno 0x%04llx %s\n",
-	    (long long) dev, diskname ? diskname : ""));
-	return 0;
-err:
-	sysfs_deinit(&cxt);
-
-	DBG(DEBUG_DEVNO,
-	    printf("failed to convert 0x%04llx to wholedisk name, errno=%d\n",
-	    (long long) dev, errno));
-	return -1;
+	return sysfs_devno_to_wholedisk( dev, diskname, len, diskdevno);
 }
 
 /*
@@ -450,7 +297,7 @@ int blkid_driver_has_major(const char *drvname, int major)
 	char buf[128];
 	int match = 0;
 
-	f = fopen(_PATH_PROC_DEVICES, "r");
+	f = fopen(_PATH_PROC_DEVICES, "r" UL_CLOEXECSTR);
 	if (!f)
 		return 0;
 
@@ -461,7 +308,7 @@ int blkid_driver_has_major(const char *drvname, int major)
 
 	while (fgets(buf, sizeof(buf), f)) {
 		int maj;
-		char name[64];
+		char name[64 + 1];
 
 		if (sscanf(buf, "%d %64[^\n ]", &maj, name) != 2)
 			continue;
@@ -474,7 +321,7 @@ int blkid_driver_has_major(const char *drvname, int major)
 
 	fclose(f);
 
-	DBG(DEBUG_DEVNO, printf("major %d %s associated with '%s' driver\n",
+	DBG(DEVNO, blkid_debug("major %d %s associated with '%s' driver",
 			major, match ? "is" : "is NOT", drvname));
 	return match;
 }
@@ -489,7 +336,7 @@ int main(int argc, char** argv)
 	dev_t	devno, disk_devno;
 	const char *errmsg = "Couldn't parse %s: %s\n";
 
-	blkid_init_debug(DEBUG_ALL);
+	blkid_init_debug(BLKID_DEBUG_ALL);
 	if ((argc != 2) && (argc != 3)) {
 		fprintf(stderr, "Usage:\t%s device_number\n\t%s major minor\n"
 			"Resolve a device number to a device name\n",
@@ -520,7 +367,9 @@ int main(int argc, char** argv)
 	free(devname);
 
 	printf("Looking for whole-device for 0x%04llx\n", (long long)devno);
-	blkid_devno_to_wholedisk(devno, diskname, sizeof(diskname), &disk_devno);
+	if (blkid_devno_to_wholedisk(devno, diskname,
+				sizeof(diskname), &disk_devno) == 0)
+		printf("found devno 0x%04llx as /dev/%s\n", (long long) disk_devno, diskname);
 
 	return 0;
 }
