@@ -32,9 +32,9 @@ struct libmnt_lock {
 	char	*linkfile;	/* path to link file (e.g. /etc/mtab~.<id>) */
 	int	lockfile_fd;	/* lock file descriptor */
 
-	int	locked : 1;	/* do we own the lock? */
-	int	sigblock : 1;	/* block signals when locked */
-	int	simplelock : 1;	/* use flock rather than normal mtab lock */
+	unsigned int	locked :1,	/* do we own the lock? */
+			sigblock :1,	/* block signals when locked */
+			simplelock :1;	/* use flock rather than normal mtab lock */
 
 	sigset_t oldsigmask;
 };
@@ -53,8 +53,7 @@ struct libmnt_lock *mnt_new_lock(const char *datafile, pid_t id)
 	char *lo = NULL, *ln = NULL;
 	size_t losz;
 
-	if (!datafile)
-		return NULL;
+	assert(datafile);
 
 	/* for flock we use "foo.lock, for mtab "foo~"
 	 */
@@ -138,6 +137,10 @@ int mnt_lock_use_simplelock(struct libmnt_lock *ml, int enable)
 	ml->simplelock = enable ? 1 : 0;
 
 	sz = strlen(ml->lockfile);
+	assert(sz);
+
+	if (sz < 1)
+		return -EINVAL;
 
 	/* Change lock name:
 	 *
@@ -298,7 +301,7 @@ static int mnt_wait_mtab_lock(struct libmnt_lock *ml, struct flock *fl, time_t m
  *
  * Where does the link point to? Obvious choices are mtab and mtab~~.
  * HJLu points out that the latter leads to races. Right now we use
- * mtab~.<pid> instead.
+ * mtab~.pid instead.
  *
  *
  * The original mount locking code has used sleep(1) between attempts and
@@ -319,7 +322,7 @@ static int mnt_wait_mtab_lock(struct libmnt_lock *ml, struct flock *fl, time_t m
  * backwardly compatible code.
  *
  * Don't forget that this code has to be compatible with 3rd party mounts
- * (/sbin/mount.<foo>) and has to work with NFS.
+ * (/sbin/mount.foo) and has to work with NFS.
  * -- kzak@redhat.com [May-2009]
  */
 
@@ -392,7 +395,7 @@ static int lock_mtab(struct libmnt_lock *ml)
 		sigprocmask(SIG_BLOCK, &sigs, &ml->oldsigmask);
 	}
 
-	i = open(linkfile, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR);
+	i = open(linkfile, O_WRONLY|O_CREAT|O_CLOEXEC, S_IRUSR|S_IWUSR);
 	if (i < 0) {
 		/* linkfile does not exist (as a file) and we cannot create it.
 		 * Read-only or full filesystem? Too many files open in the system?
@@ -424,7 +427,7 @@ static int lock_mtab(struct libmnt_lock *ml)
 				rc = -errno;
 			goto failed;
 		}
-		ml->lockfile_fd = open(lockfile, O_WRONLY);
+		ml->lockfile_fd = open(lockfile, O_WRONLY|O_CLOEXEC);
 
 		if (ml->lockfile_fd < 0) {
 			/* Strange... Maybe the file was just deleted? */
@@ -557,7 +560,7 @@ void increment_data(const char *filename, int verbose, int loopno)
 	FILE *f;
 	char buf[256];
 
-	if (!(f = fopen(filename, "r")))
+	if (!(f = fopen(filename, "r" UL_CLOEXECSTR)))
 		err(EXIT_FAILURE, "%d: failed to open: %s", getpid(), filename);
 
 	if (!fgets(buf, sizeof(buf), f))
@@ -566,7 +569,7 @@ void increment_data(const char *filename, int verbose, int loopno)
 	fclose(f);
 	num = atol(buf) + 1;
 
-	if (!(f = fopen(filename, "w")))
+	if (!(f = fopen(filename, "w" UL_CLOEXECSTR)))
 		err(EXIT_FAILURE, "%d: failed to open: %s", getpid(), filename);
 
 	fprintf(f, "%ld", num);
@@ -585,7 +588,7 @@ void clean_lock(void)
 	mnt_free_lock(lock);
 }
 
-void sig_handler(int sig)
+void __attribute__((__noreturn__)) sig_handler(int sig)
 {
 	errx(EXIT_FAILURE, "\n%d: catch signal: %s\n", getpid(), strsignal(sig));
 }
