@@ -20,6 +20,17 @@ function ts_abspath {
 	pwd
 }
 
+function ts_canonicalize {
+	P="$1"
+	C=$(readlink -f $P)
+
+	if [ -n "$C" ]; then
+		echo "$C"
+	else
+		echo "$P"
+	fi
+}
+
 function ts_skip_subtest {
 	echo " IGNORE ($1)"
 }
@@ -75,6 +86,12 @@ function ts_has_option {
 	echo -n $ALL | sed 's/ //g' | awk 'BEGIN { FS="="; RS="--" } /('$NAME'$|'$NAME'=)/ { print "yes" }'
 }
 
+function ts_option_argument {
+	NAME="$1"
+	ALL="$2"
+	echo -n $ALL | sed 's/ //g' | awk 'BEGIN { FS="="; RS="--" } /'$NAME'=/ { print $2 }'
+}
+
 function ts_init_core_env {
 	TS_NS="$TS_COMPONENT/$TS_TESTNAME"
 	TS_OUTPUT="$TS_OUTDIR/$TS_TESTNAME"
@@ -109,7 +126,26 @@ function ts_init_env {
 
 	export LANG LANGUAGE LC_ALL CHARSET
 
+	mydir=$(ts_canonicalize "$mydir")
+
+	# automake directories
+	top_srcdir=$(ts_option_argument "srcdir" "$*")
+	top_builddir=$(ts_option_argument "builddir" "$*")
+
+	# where is this script
 	TS_TOPDIR=$(ts_abspath $mydir/../../)
+
+	# default
+	if [ -z "$top_srcdir" ]; then
+		top_srcdir="$TS_TOPDIR/.."
+	fi
+	if [ -z "$top_builddir" ]; then
+		top_builddir="$TS_TOPDIR/.."
+	fi
+
+	top_srcdir=$(ts_abspath $top_srcdir)
+	top_builddir=$(ts_abspath $top_builddir)
+
 	TS_SCRIPT="$mydir/$(basename $0)"
 	TS_SUBDIR=$(dirname $TS_SCRIPT)
 	TS_TESTNAME=$(basename $TS_SCRIPT)
@@ -120,8 +156,8 @@ function ts_init_env {
 
 	TS_SELF="$TS_SUBDIR"
 
-	TS_OUTDIR="$TS_TOPDIR/output/$TS_COMPONENT"
-	TS_DIFFDIR="$TS_TOPDIR/diff/$TS_COMPONENT"
+	TS_OUTDIR="$top_builddir/tests/output/$TS_COMPONENT"
+	TS_DIFFDIR="$top_builddir/tests/diff/$TS_COMPONENT"
 
 	ts_init_core_env
 
@@ -129,7 +165,6 @@ function ts_init_env {
 
 	BLKID_FILE="$TS_OUTDIR/${TS_TESTNAME}.blkidtab"
 
-
 	declare -a TS_SUID_PROGS
 	declare -a TS_SUID_USER
 	declare -a TS_SUID_GROUP
@@ -222,6 +257,9 @@ function ts_gen_diff {
 
 	if [ -s "$TS_OUTPUT" ]; then
 
+		# remove libtool lt- prefixes
+		sed --in-place 's/^lt\-\(.*\: \)/\1/g' $TS_OUTPUT
+
 		[ -d "$TS_DIFFDIR" ] || mkdir -p "$TS_DIFFDIR"
 		diff -u $TS_EXPECTED $TS_OUTPUT > $TS_DIFF
 
@@ -377,7 +415,7 @@ function ts_device_has_uuid {
 }
 
 function ts_is_mounted {
-	local DEV=$1
+	local DEV=$(ts_canonicalize "$1")
 
 	grep -q $DEV /proc/mounts && return 0
 
@@ -387,21 +425,6 @@ function ts_is_mounted {
 	return 1
 }
 
-function ts_swapoff {
-	local DEV="$1"
-
-	# swapoff doesn't exist in build tree
-	if [ ! -x "$TS_CMD_SWAPOFF" ]; then
-		ln -sf $TS_CMD_SWAPON $TS_CMD_SWAPOFF
-		REMSWAPOFF="true"
-	fi
-	LD_LIBRARY_PATH="$U_L_LIBRARY_PATH" \
-			$TS_CMD_SWAPOFF $DEV 2>&1 >> $TS_OUTPUT
-	if [ -n "$REMSWAPOFF" ]; then
-		rm -f $TS_CMD_SWAPOFF
-	fi
-}
-
 function ts_fstab_open {
 	echo "# <!-- util-linux test entry" >> /etc/fstab
 }
@@ -438,10 +461,35 @@ s/# <!-- util-linux.*-->//;
 }
 
 function ts_fdisk_clean {
+	local DEVNAME=$(basename "$1")
+
 	# remove non comparable parts of fdisk output
-	[ x"${DEVNAME}" != x"" ] && sed -i -e "s/\/dev\/${DEVNAME}/\/dev\/.../g" $TS_OUTPUT
+	if [ x"${DEVNAME}" != x"" ]; then
+	       sed -i -e "s/\/dev\/${DEVNAME}/\/dev\/.../g" $TS_OUTPUT
+	fi
+
 	sed -i -e 's/Disk identifier:.*//g' \
 	       -e 's/Building a new.*//g' \
 	       -e 's/Welcome to fdisk.*//g' \
 	       $TS_OUTPUT
 }
+
+function ts_scsi_debug_init {
+
+	modprobe --dry-run --quiet scsi_debug
+	[ "$?" == 0 ] || ts_skip "missing scsi_debug module"
+
+	rmmod scsi_debug &> /dev/null
+	modprobe scsi_debug $*
+	[ "$?" == 0 ] || ts_die "Cannot init device"
+
+	DEVNAME=$(grep scsi_debug /sys/block/*/device/model | awk -F '/' '{print $4}')
+	[ "x${DEVNAME}" == "x" ] && ts_die "Cannot find device"
+
+	DEVICE="/dev/${DEVNAME}"
+
+	sleep 1
+	udevadm settle
+
+	echo $DEVICE
+}
